"use strict";
/* eslint-disable @typescript-eslint/no-require-imports, global-require */
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
module.exports = helper_plugin_utils_1.declare((api, options) => {
    const env = process.env.NODE_ENV;
    const isEnvDevelopment = env === 'development';
    const isEnvProduction = env === 'production';
    const isEnvTest = env === 'test';
    if (!isEnvDevelopment && !isEnvProduction && !isEnvTest) {
        throw new Error(`Using \`@justia/babel-preset\` requires that you specify \`NODE_ENV\` or \`BABEL_ENV\` environment variables. Valid values are "development", "test", and "production". Instead, received: ${JSON.stringify(env)}.`);
    }
    // see docs about api at:
    // https://babeljs.io/docs/en/config-files#apicache
    api.assertVersion('^7.0.0');
    const { targets = {}, modules = 'auto', useBuiltIns = false, corejs = 2, looseClasses = false, runtimeVersion, runtimeHelpersUseESModules = !modules, transformRuntime = true, include = [], exclude = [] } = options;
    if (typeof modules !== 'undefined' && typeof modules !== 'boolean' && modules !== 'auto') {
        throw new TypeError('`@justia/babel-preset` only accepts `true`, `false`, or `"auto"` as the value of the "modules" option');
    }
    const toExclude = [
        ...new Set(['transform-async-to-generator', 'transform-template-literals', 'transform-regenerator', ...exclude])
    ];
    const isTypeScriptEnabled = Boolean(options.typescript);
    const debug = typeof options.debug === 'boolean' ? options.debug : false;
    const development = typeof options.development === 'boolean' ? options.development : api.cache.using(() => isEnvDevelopment);
    return {
        presets: [
            [
                require('@babel/preset-env'),
                {
                    targets,
                    modules: modules === false ? false : 'auto',
                    debug,
                    include,
                    exclude: toExclude,
                    useBuiltIns,
                    corejs,
                    shippedProposals: options.shippedProposals === true
                }
            ],
            [require('@babel/preset-react'), { development }],
            isTypeScriptEnabled && [require('@babel/preset-typescript')]
        ].filter(Boolean),
        plugins: [
            // Turn on legacy decorators for TypeScript files
            isTypeScriptEnabled && [require('@babel/plugin-proposal-decorators'), false],
            looseClasses
                ? [
                    require('@babel/plugin-transform-classes'),
                    {
                        loose: true
                    }
                ]
                : null,
            [
                require('@babel/plugin-transform-template-literals'),
                {
                    spec: true
                }
            ],
            require('@babel/plugin-transform-property-mutators'),
            require('@babel/plugin-transform-member-expression-literals'),
            require('@babel/plugin-transform-property-literals'),
            require('@babel/plugin-proposal-nullish-coalescing-operator'),
            require('@babel/plugin-proposal-numeric-separator'),
            require('@babel/plugin-proposal-optional-catch-binding'),
            require('@babel/plugin-proposal-optional-chaining'),
            [
                require('@babel/plugin-proposal-object-rest-spread'),
                {
                    useBuiltIns: true
                }
            ],
            transformRuntime
                ? [
                    require('@babel/plugin-transform-runtime'),
                    {
                        absoluteRuntime: false,
                        corejs: false,
                        helpers: true,
                        regenerator: false,
                        useESModules: runtimeHelpersUseESModules,
                        version: runtimeVersion
                    }
                ]
                : null
        ].filter(Boolean),
        overrides: [
            isTypeScriptEnabled && {
                test: /\.tsx?$/,
                plugins: [[require('@babel/plugin-proposal-decorators'), { legacy: true }]]
            }
        ].filter(Boolean)
    };
});
